<?php
namespace Controller;

use Service\DeleteCascadeService;
use Service\Traits\TResponse;
use Util\Utilities;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;


class Hierarchy extends Crud{
    const MESSAGE_NO_HIERARCHY_GENERATED = "Cannot match structures to generate a hierarchy.";

    protected $dataSourceName = 'hierarchy';
    protected $hierarchyService;
    protected $utilities;

    use TResponse;

    public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService, \Service\Hierarchy $hierarchy, Utilities $utilities) {
        parent::__construct($mongoManager, $deleteCascadeService);
        $this->hierarchyService = $hierarchy;
        $this->utilities = $utilities;
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
    }

    public function findNext (DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();
        $hierarchies = $this->hierarchyService->findNext($params);
        $strucures = $this->getStrucutresByHierarchies($hierarchies)->getRows();
        $indexedStructure = $this->hierarchyService->indexByMongoId($strucures);
        $hierarchies = $this->hierarchyService->relateHierarchyWithStructureType($hierarchies, $indexedStructure);
        $hierarchies = Utilities::parseDataSourceZeedhiNext($hierarchies, $this->count($params));
        $hierarchies["pagination"]["page"] = isset($params["page"]) ? (int)$params["page"] : 1;
        $hierarchies["pagination"]["limit"] = isset($params["limit"]) ? (int)$params["limit"] : 9999;
        // $this->sendResponse(Utilities::parseDataSourceZeedhiNext($hierarchies), 200);
        $this->sendResponse($hierarchies, 200);
    }
    /* $dataSet = Utilities::parseDataSourceZeedhiNext($decodedStructures, $this->count($params));
    $dataSet["pagination"]["page"] = isset($params["page"]) ? (int)$params["page"] : 1;
    $dataSet["pagination"]["limit"] = isset($params["limit"]) ? (int)$params["limit"] : 9999;
    $this->sendResponse($dataSet, 200); */
    public function count($params) {
        return $this->mongoManager->countNext($this->dataSourceName, $params);
    }

     public function find (DTO\Request\Filter $request, DTO\Response $response) {
        parent::find($request, $response);
        $dataSet = $response->getDataSets()[0];
        $hierarchies = $dataSet->getRows();
        $strucures = $this->getStrucutresByHierarchies($hierarchies)->getRows();
        $indexedStructures = $this->utilities->indexByMongoId($strucures);
        $hierarchies = $this->hierarchyService->relateHierarchyWithStructureType($hierarchies, $indexedStructures);
        $newDataSet = new DataSet($this->dataSourceName, $hierarchies);
        $response->setDataSets(array($newDataSet));
    }

    public function deleteAll(DTO\Request $request, DTO\Response $response) {
		$params = $request->getParameters();
		$this->hierarchyService->deleteAll($params);
	}

    public function generateHierarchies (DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $hierarchyType = $row['_id'];
        $parentStructures = $this->findStructuresByStructureType($row['typeParent']);
        $childsStructures = $this->findStructuresByStructureType($row['typeChild']);
        $fieldKeysRelation = $row['fieldKeysRelation'];
        $rowsToInsert = array();
        foreach($parentStructures as $parentStructure) {
            $rowsToInsert = array_merge($rowsToInsert, $this->hierarchyService->generateHierarchy($parentStructure, $childsStructures, $hierarchyType, $fieldKeysRelation));
        }
        if (count($rowsToInsert) == 0) {
            $response->addMessage(new DTO\Response\Message(self::MESSAGE_NO_HIERARCHY_GENERATED, DTO\Response\Message::TYPE_WARNING));
        }
        else {
            $dataSet = new DataSet($this->dataSourceName, $rowsToInsert);
            $this->saveHierarchy($dataSet, $response);
        }
    }

    protected function saveHierarchy (DataSet $dataSet, DTO\Response $response) {
        try {
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->mongoManager->persist($dataSet);
            $nrPersistedRows = count($persistedRows);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $response->addNotification(new DTO\Response\Notification("{$nrPersistedRows} row(s) persisted with success"));
        } catch (\Exception $e) {
            echo($e->getTraceAsString());
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function findStructuresByStructureType($structureTypeId, $maxLines = null) {
        $filterCriteria = new FilterCriteria('structure', array(), null, $maxLines);
        $filterCriteria->addCondition('type', FilterCriteria::EQ, $structureTypeId);
        return $this->mongoManager->findBy($filterCriteria)->getRows();
    }

    public function getStrucutresByHierarchies ($hierarchies) {
        $usedIds = $this->hierarchyService->getStructureIdsUsed($hierarchies);
        $numberOfExpectedLines = count($usedIds);
        return $this->findStructuresById($usedIds, $numberOfExpectedLines);
    }

    protected function findStructuresById($structureIds, $maxLines = null) {
        $filterCriteria = new FilterCriteria('structure', array(), null, $maxLines);
        $filterCriteria->addCondition('_id', FilterCriteria::IN, $structureIds);
        return $this->mongoManager->findBy($filterCriteria);
    }

}